/*
This file is part of Gnome Photo Printer.

    Gnome Photo Printer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gnome Photo Printer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gnome Photo Printer; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GPP_CALLBACKS_H
#define GPP_CALLBACKS_H	1

void
drag_received (GtkWidget * widget, GdkDragContext * context, gint x, gint y,
	       GtkSelectionData * data, guint info, guint time, gpointer
	       userdata);

void func_gpp_quit (GtkWidget * widget, gpointer data);
void func_gpp_remove (GtkWidget * widget, gpointer view);
void func_gpp_add (GtkWidget * widget, gpointer data);
void func_gpp_add_ok (GtkWidget * widget, gpointer view);
void func_gpp_add_cancel (GtkWidget * widget, gpointer data);
void func_gpp_preview (GtkWidget * widget, gpointer progress_bar);
void func_gpp_print (GtkWidget * widget, gpointer data);

void func_gpp_layout (GtkWidget * widget, gpointer data);

void func_gpp_image_1 (GtkWidget * widget, gpointer data);
void func_gpp_image_2 (GtkWidget * widget, gpointer data);

#endif
